/**
 * Created with IntelliJ IDEA.
 * User: zyc
 * Date: 13-5-6
 * Time: 上午10:58
 * To change this template use File | Settings | File Templates.
 */
package com {
import flash.display.DisplayObject;
import flash.display.Sprite;
import flash.events.MouseEvent;
import flash.geom.Point;

import mx.core.Container;

import spark.components.Image;

public class ObjectHandler{

    //对象的容器
    private var container:Container;

    //实际控制对象
    private var widget: DisplayObject;

    private var displayEl: Image;

    //4个可拖动框
    private var rects:Array = [];

    //当前正在操作的句柄
    private var currRect:Sprite;

    // 拖动缩放大小
    private var deltaScale:Number;

    private var startX:Number;
    private var startY:Number;

    /**
     * 构造方法
     * @param container
     */
    public function ObjectHandler(container: Container) {
        this.container = container;
        rects = [];
        for (var i:int = 0; i < 4; i++) {
            var rect:Sprite = createRect();
            rects[i] = rect;

            rect.addEventListener(MouseEvent.MOUSE_DOWN, rect_mouse_down);
        }
        container.addEventListener(MouseEvent.MOUSE_UP, rect_mouse_up)
    }
    // 在句柄上按下鼠标左键
    private function rect_mouse_down(e:MouseEvent):void {
        trace("evt_rect_mouse_down");
        currRect = e.currentTarget as Sprite;
        if (!currRect) {
            return;
        }
        container.addEventListener(MouseEvent.MOUSE_MOVE, mouse_move);

        startX = e.stageX;
        startY = e.stageY;

        // 实际操作对象
        displayEl = cloneImage(widget as Image, widget.width, widget.height);
        displayEl.alpha = 0.5;
        displayEl.width = widget.width * widget.scaleX;
        displayEl.height = widget.height * widget.scaleY;
        displayEl.x = rects[0].x;
        displayEl.y = rects[0].y;

        container.addChild(displayEl);

        var index:int = rects.indexOf(currRect);

        setTransformCenter(displayEl, index);
    }

    // 拖动矩形句柄，鼠标移动
    private function mouse_move(e:MouseEvent):void {
        if (!currRect) {
            return;
        }
        trace("evt_mouse_move");
        var moveX:int = e.stageX - startX;
        var moveY:int = e.stageY - startY;

        var ind:int = rects.indexOf(currRect);

        // 正代表放大，负代表缩小
        if (ind == 0) {
            moveX = -moveX;
            moveY = -moveY;
        } else if (ind == 1) {
            moveY = -moveY;
        } else if (ind == 2) {
            moveX = -moveX;
        }

        deltaScale = Math.max(  moveY / widget.height, moveX / widget.width );
        displayEl.scaleX = deltaScale / widget.scaleX + 1;
        displayEl.scaleY = deltaScale / widget.scaleY + 1;
    }

    // 在句柄上松开鼠标左键
    private function rect_mouse_up(e:MouseEvent):void {
        trace("rect_mouse_up");
        if(!currRect) return;

        fixPosition();

        currRect = null;
        container.removeEventListener(MouseEvent.MOUSE_MOVE, mouse_move);
        container.removeChild(displayEl);

        widget.scaleX += deltaScale || 0;
        widget.scaleY += deltaScale || 0;

        setRects();
    }

    // 设置不动点
    private static function setTransformCenter(el:Image,index:int):void{
        if(!el) return;
        switch (index){
            case 0:
                el.transformX = el.width;
                el.transformY = el.height;
                break;
            case 1:
                el.transformX = 0;
                el.transformY = el.height;
                break;
            case 2:
                el.transformX = el.width;
                el.transformY = 0;
                break;
            case 3:
                el.transformX = 0;
                el.transformY = 0;
                break;
        }
    }

    //
    private function fixPosition():void{
        var index:int = rects.indexOf(currRect);
        var scale: Number = widget.scaleX + (deltaScale || 0);

        var w:Number = widget.width * scale;
        var h:Number = widget.height * scale;

        var left:Number, top:Number;
        switch (index){
            case 0:
                left = rects[3].x - w;
                top = rects[3].y - h;
                break;
            case 1:
                left = rects[2].x;
                top = rects[2].y - h;
                break;
            case 2:
                left = rects[1].x - w;
                top = rects[1].y;
                break;
            case 3:
                left = rects[0].x;
                top = rects[0].y;
                break;
        }
        widget.x = left;
        widget.y = top;
    }

    public function initRender(_widget:DisplayObject):void{
        if(widget){
            return;
        }

        // 原始对象
        widget = _widget;

        // 设置句柄
        setRects();

        // 设置边框
        setBorder();
    }

    public function clear():void{
        var handler:Image;
        for (var i:int = 0; i < rects.length; i++) {
            handler = rects[i];
            if (handler.parent) {
                handler.parent.removeChild(handler);
            }
        }
        //清除边线
        if(widget){
            var imgs:Image = widget as Image;
            imgs.graphics.clear();
            widget = null;
        }
    }
    /**
     * 复制 图片
     * @param el Spark 的Image组件
     * @param w
     * @param h
     * @return
     */
    public static function cloneImage(el:Image, w:Number, h:Number):Image {
        var img:Image = new Image();
        img.source = el.source;
        img.width = w;
        img.height = h;
        return img;
    }

    // 创建句柄，用来拖动和拉伸对象
    private static function createRect():Image {
        var img:Image = new Image();
        var size:int = 10;
        img.graphics.beginFill(0xcf67b9, 0.5);
        img.graphics.drawRect(-size / 2, -size / 2, size, size);
        img.graphics.endFill();
        img.graphics.lineStyle(1, 0);
        img.graphics.drawRect(-size / 2, -size / 2, size, size);
        return img;
    }

    // 绘制边线
    private function setBorder():void {
        var img:Image = widget as Image;
        img.graphics.clear();
        img.graphics.lineStyle(6, 0xcf67b9, 0.5);
        img.graphics.moveTo(0, 0);
        img.graphics.lineTo(img.width, 0);
        img.graphics.lineTo(img.width, img.height);
        img.graphics.lineTo(0, img.height);
        img.graphics.lineTo(0, 0);
    }

    // 设定各个句柄的位置
    private function setRects():void {
        var x:Number, y:Number, w:Number, h:Number;

        w = widget.width * widget.scaleX;
        h = widget.height * widget.scaleY;

        x = widget.x;
        y = widget.y;

        var p1:Point = new Point(x,y);
        var p2:Point = new Point(x + w, y + h);

        rects[0].x = p1.x; //左上
        rects[0].y = p1.y;

        rects[1].x = p2.x; //右上
        rects[1].y = p1.y;

        rects[2].x = p1.x; //左下
        rects[2].y = p2.y;

        rects[3].x = p2.x; //右下
        rects[3].y = p2.y;

        for (var i:int = 0; i < rects.length; i++) {
            container.addChild(rects[i]);
        }
    }
}
}